==> kaya_0.2.7.orig.tar.gz: kaya-0.2.7/rts/Closure.cc <==
/**
    Kaya run-time system
    Copyright (C) 2004, 2005 Edwin Brady

    This file is distributed under the terms of the GNU Lesser General
    Public Licence. See COPYING for licence.
*/

using namespace std;

#include <iostream>

#include "Closure.h"
#include "VMState.h"
#include "stdfuns.h"

Closure::Closure(VMState* vm, func fn, kint args, kint arity, bool stack):m_fn(fn),m_arity(arity)
{
//    cout << "Making closure " << (void*)(m_fn) << endl;
    m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*arity);
    m_numargs=args;
    if (stack) {
	for(kint i=0;i<args;++i) {
//	cout << "Arg" << endl;
/// Create a *new* reference, or things break in confusing ways.
	  m_args[i]=vm->doPop()->clone();
//	m_args[i]=vm->doPop();
	}
    }
}

Closure::Closure(Closure& x) {
    // Shallow copy everything, we just need a new argument list.
    m_numargs = x.m_numargs;
    m_arity = x.m_arity;
    m_fn = x.m_fn;
    m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_arity);
    for(kint i=0;i<m_numargs;++i) {
	// Make it a new pointer, so that we don't modify inside it.
	m_args[i]=x.m_args[i]->clone();
    }
}

void Closure::addArgs(VMState* vm, kint added)
{
    kint newnum = m_numargs+added;
    kint i;

    // Better make sure there's enough room! (it really should have been
    // allocated, but there may be circumstances where it hasn't, if someone
    // is abusing the type system...)
    if (newnum>m_arity) {
	m_args = (Value**)GC_REALLOC(m_args, newnum);
    }

    for(i=m_numargs; i<newnum; ++i) {
        m_args[i]=vm->doPop()->clone();
    }
    m_numargs = newnum;
}

Closure::~Closure()
{
    GC_FREE(m_args);
}

void Closure::run(VMState* vm)
{
    for(kint i=m_numargs;i>0;i--) {
	vm->push(m_args[i-1]);
    }
    m_fn(vm);
}

bool Closure::eq(Closure* c, map<Value*, Value*>& done)
{
    if (getFnID()!=c->getFnID()) { return false; }
    if (m_numargs!=c->m_numargs) { return false; }

    for(kint i=0;i<m_numargs;++i)
    {
	if (!funtable_eq_aux(m_args[i],c->m_args[i],done)) { return false; }
    }
    return true;

}

Closure* Closure::copy(map<Value*, Value*>& done)
{
    Closure* c = new Closure(NULL,m_fn,m_numargs,m_arity,false);
    for(kint i=0;i<m_numargs;++i) {
	Value* copied;
	Value* x = m_args[i];
	// if x is already copied, return the copied value
	if (done.find(x)!=done.end()) {
	    copied = done[x];
	}
	else {
	    copied = funtable_copy_aux(x, done);
	}
	c->setArg(i,copied);
    }
    return c;
}

==> kaya_0.4.4.orig.tar.gz: kaya-0.4.4/rts/Closure.cc <==
/**
    Kaya run-time system
    Copyright (C) 2004, 2005 Edwin Brady

    This file is distributed under the terms of the GNU Lesser General
    Public Licence. See COPYING for licence.
*/

using namespace std;

#include <iostream>

#include "Closure.h"
#include "VMState.h"
#include "stdfuns.h"

Closure::Closure(VMState* vm, func fn, kint args, kint arity, 
		 bool lam, bool stack):m_fn(fn),m_arity(arity)
{
//    cout << "Making closure " << (void*)(m_fn) << endl;
    m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*arity);
    m_numargs=args;
    if (stack) {
	for(kint i=0;i<args;++i) {
//	cout << "Arg" << endl;
// If it's a lambda, we want the thing to be in the same scope, so the 
// variable ought to live at the same address. Otherwise, make a copy so
// that modifying it won't modify it in the calling scope.
	    if (lam) {
		m_args[i]=vm->doPop();
	    } else {
		m_args[i]=vm->doPop()->cloneTo(vm->getVPool());
	    }
	}
    }
}

Closure::Closure(Closure& x) {
    // Shallow copy everything, we just need a new argument list.
    m_numargs = x.m_numargs;
    m_arity = x.m_arity;
    m_fn = x.m_fn;
    m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_arity);
    for(kint i=0;i<m_numargs;++i) {
	// Make it a new pointer, so that we don't modify inside it.
	m_args[i]=x.m_args[i]->clone();
    }
}

void Closure::addArgs(VMState* vm, kint added, bool lam)
{
    kint newnum = m_numargs+added;
    kint i;

    // Better make sure there's enough room! (it really should have been
    // allocated, but there may be circumstances where it hasn't, if someone
    // is abusing the type system...)
    if (newnum>m_arity) {
	m_args = (Value**)GC_REALLOC(m_args, newnum);
    }

    for(i=m_numargs; i<newnum; ++i) {
	if (lam) {
	    m_args[i]=vm->doPop();
	} else {
	    m_args[i]=vm->doPop()->cloneTo(vm->getVPool());
	}
    }
    m_numargs = newnum;
}

Closure::~Closure()
{
    GC_FREE(m_args);
}

void Closure::run(VMState* vm)
{
    for(kint i=m_numargs;i>0;i--) {
	vm->push(m_args[i-1]);
    }
    m_fn(vm);
}

bool Closure::eq(Closure* c, map<Value*, Value*>& done)
{
    if (getFnID()!=c->getFnID()) { return false; }
    if (m_numargs!=c->m_numargs) { return false; }

    for(kint i=0;i<m_numargs;++i)
    {
	if (!funtable_eq_aux(m_args[i],c->m_args[i],done)) { return false; }
    }
    return true;

}

Closure* Closure::copy(VMState* vm, map<Value*, Value*>& done)
{
    Closure* c = new Closure(NULL,m_fn,m_numargs,m_arity,false);
    for(kint i=0;i<m_numargs;++i) {
	Value* copied;
	Value* x = m_args[i];
	// if x is already copied, return the copied value
	if (done.find(x)!=done.end()) {
	    copied = done[x];
	}
	else {
	    copied = funtable_copy_aux(vm, x, done);
	}
	c->setArg(i,copied);
    }
    return c;
}
